/*
 * Soubor:  types.c
 * Datum:   08.10.2007 08:16
 * Autor:   David Martinek, martinek@fit.vutbr.cz
 * Projekt: Types
 * Popis:   Vytiskne tabulku minimlnch a maximlnch hodnot vech celoselnch
 *          datovch typ. Demonstruje, jak lze tyto hodnoty vypotat, ppadn
 *          zskat pomoc limits.h.
 */

#include <stdio.h>
#include <stdlib.h>

#include <limits.h>

/**
 * Vytiskne tabulku minimlnch a maximlnch hodnot pomoc bitovch operac.
 */
void typesTable(void)
{
  unsigned char ucharMax = (unsigned char)~0;
  unsigned short ushrtMax = (unsigned short)~0;
  unsigned int uintMax = (unsigned int)~0;
  unsigned long ulongMax = (unsigned long)~0;
  unsigned long long ullongMax = (unsigned long long)~0;

  char charMax = ucharMax >> 1; 
  // je nutn pracovat s promnnou typu unsigned, jinak by bitov posun doploval
  // zleva znamnkov bit
  // alternativn:
  // char charMax = (unsigned char)~0 >> 1;
  char charMin = ~charMax;
  short shrtMax = ushrtMax >> 1;
  short shrtMin = ~shrtMax;
  int intMax = uintMax >> 1;
  int intMin = ~intMax;
  long longMax = ulongMax >> 1;
  long longMin = ~longMax;
  long long llongMax = ullongMax >> 1;
  long long llongMin = ~llongMax;


  printf("Velikosti typ pomoc bitovch operac:\n");
  printf("TYP              SIZE %20s %20s\n", "MINIMUM", "MAXIMUM");
  printf("char                %d %20hhd %20hhd\n", 
      sizeof(char), charMin, charMax);
  printf("short               %d %20hd %20hd\n", 
      sizeof(short), shrtMin, shrtMax);
  printf("int                 %d %20d %20d\n", 
      sizeof(int), intMin, intMax);
  printf("long                %d %20ld %20ld\n", 
      sizeof(long), longMin, longMax);
  printf("long long           %d %20lld %20lld\n", 
      sizeof(long long), llongMin, llongMax);

  printf("unsigned char       %d %20hhu %20hhu\n", 
      sizeof(unsigned char), (unsigned char)0, ucharMax);
  printf("unsigned short      %d %20hu %20hu\n", 
      sizeof(unsigned short), (unsigned short)0, ushrtMax);
  printf("unsigned int        %d %20u %20u\n", 
      sizeof(unsigned int), (unsigned int)0, uintMax);
  printf("unsigned long       %d %20lu %20lu\n", 
      sizeof(unsigned long), (unsigned long)0, ulongMax);
  printf("unsigned long long  %d %20llu %20llu\n", 
      sizeof(unsigned long long), (unsigned long long)0, ullongMax);
 
}

/**
 * Vytiskne tabulku pomoc konstant z limits.h
 */
void typesTableLimits(void)
{
  printf("Velikosti typ pomoc limits.h:\n");
  printf("TYP              SIZE %20s %20s\n", "MINIMUM", "MAXIMUM");
  printf("char                %d %20hhd %20hhd\n", 
      sizeof(char), CHAR_MIN, CHAR_MAX);
  printf("short               %d %20hd %20hd\n", 
      sizeof(short), SHRT_MIN, SHRT_MAX);
  printf("int                 %d %20d %20d\n", 
      sizeof(int), INT_MIN, INT_MAX);
  printf("long                %d %20ld %20ld\n", 
      sizeof(long), LONG_MIN, LONG_MAX);
  printf("long long           %d %20lld %20lld\n", 
      sizeof(long long), LLONG_MIN, LLONG_MAX);

  printf("unsigned char       %d %20hhu %20hhu\n", 
      sizeof(unsigned char), (unsigned char)0, UCHAR_MAX);
  printf("unsigned short      %d %20hu %20hu\n", 
      sizeof(unsigned short), (unsigned short)0, USHRT_MAX);
  printf("unsigned int        %d %20u %20u\n", 
      sizeof(unsigned int), (unsigned int)0, UINT_MAX);
  printf("unsigned long       %d %20lu %20lu\n", 
      sizeof(unsigned long), (unsigned long)0, ULONG_MAX);
  printf("unsigned long long  %d %20llu %20llu\n", 
      sizeof(unsigned long long), (unsigned long long)0, ULLONG_MAX);
 
}

int main(void)
{
  typesTable();
  printf("---------------------------------\n");
  typesTableLimits();

  return EXIT_SUCCESS;
}

/* konec souboru types.c */
